package ru.voskhod.smev.client.api.template.impl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.voskhod.smev.client.api.services.template.AttachmentDownloadTemplate;
import ru.voskhod.smev.client.api.services.transport.LargeAttachmentTransport;
import ru.voskhod.smev.client.api.types.message.SMEVMessage;
import ru.voskhod.smev.client.api.types.message.attachment.LargeAttachment;
import ru.voskhod.smev.client.api.types.message.attachment.MTOMAttachment;
import ru.voskhod.smev.client.api.types.message.attachment.SMEVAttachment;

public class FileAttachmentDownloadTemplateImpl implements AttachmentDownloadTemplate
{
    private static final Logger logger = LoggerFactory.getLogger(FileAttachmentDownloadTemplateImpl.class);

    private String localStorage;

    public FileAttachmentDownloadTemplateImpl(String localStorage) {
        if (localStorage == null)
            throw new NullPointerException("localStorage");
        if (localStorage.isEmpty())
            throw new IllegalArgumentException("localStorage is empty");
        this.localStorage = localStorage;
    }

    @Override
    public void download(SMEVMessage message, LargeAttachmentTransport largeAttachmentTransport) {
        if (message == null || message.getData() == null || message.getData().getAttachments() == null)
            return;
        try {
            for (SMEVAttachment attachment : message.getData().getAttachments()) {
                if (attachment instanceof MTOMAttachment) { // add folder
                    saveFile((MTOMAttachment) attachment);
                } else if (attachment instanceof LargeAttachment) { // add folder
                    saveLargeFile((LargeAttachment) attachment, largeAttachmentTransport);
                }
            }
        } catch (Exception ex) {
            throw new RuntimeException("Error for " + message.getSMEVMetadata().getMessageIdentity().getMessageId(), ex);
        }
    }

    private void saveFile(MTOMAttachment attachment) throws Exception {
        File file = new File(Paths.get(localStorage, attachment.getAttachmentId()).toUri());
        try {
            Files.copy(attachment.getContent().getInputStream(), file.toPath());
        } finally {
            try {
                attachment.getContent().getInputStream().close();
            } catch (IOException e) {
                logger.debug("Ошибка закрытия входного аттачмента", e);
            }
        }
    }

    private void saveLargeFile(LargeAttachment attachment, LargeAttachmentTransport largeAttachmentTransport) throws Exception {
        if (largeAttachmentTransport == null)
            return;
        File file = new File(Paths.get(localStorage, attachment.getContentRef().getRawPath()).toUri());
        OutputStream output = null;
        try {
            if (!file.exists()) {
                file.createNewFile();
            }
            output = new FileOutputStream(file);
            largeAttachmentTransport.download(attachment, output);
            output.flush();
        } finally {
            if (output != null) {
                try {
                    output.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
