package ru.voskhod.smev.client.api.template.impl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.Files;

import ru.voskhod.smev.client.api.services.template.AttachmentDownloadTemplate;
import ru.voskhod.smev.client.api.services.transport.LargeAttachmentTransport;
import ru.voskhod.smev.client.api.types.message.SMEVMessage;
import ru.voskhod.smev.client.api.types.message.attachment.LargeAttachment;
import ru.voskhod.smev.client.api.types.message.attachment.MTOMAttachment;
import ru.voskhod.smev.client.api.types.message.attachment.SMEVAttachment;

public class MessageIdAttachmentDownloadTemplateImpl implements AttachmentDownloadTemplate
{
    final private File storeDir;

    private static void deleteRecursively(File file) {
        if (file.isDirectory()) {
            for (File f : file.listFiles())
                deleteRecursively(f);
        }
        file.delete();
    }

    private File createDir(String messageId) {
        File dir = new File(storeDir, messageId);
        if (dir.exists())
            deleteRecursively(dir);
        dir.mkdirs();
        return dir;
    }

    private void saveFile(File dir, MTOMAttachment attachment) {
        try {
            File file = new File(dir, attachment.getAttachmentId());
            try (InputStream in = attachment.getContent().getInputStream()) {
                Files.copy(in, file.toPath());
            }
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }

    private void saveLargeFile(File dir, LargeAttachment attachment, LargeAttachmentTransport largeAttachmentTransport) throws Exception {
        File file = new File(dir, attachment.getContentRef().getRawPath());
        try (OutputStream output = new FileOutputStream(file)) {
            largeAttachmentTransport.download(attachment, output);
            output.flush();
        }
    }

    @Override
    public void download(SMEVMessage message, LargeAttachmentTransport largeAttachmentTransport) {
        if (message == null || message.getData() == null || message.getData().getAttachments() == null || message.getData().getAttachments().isEmpty())
            return;
        try {
            File dir = createDir(message.getSMEVMetadata().getMessageIdentity().getMessageId());
            for (SMEVAttachment attachment : message.getData().getAttachments()) {
                if (attachment instanceof MTOMAttachment) { // add folder
                    saveFile(dir, (MTOMAttachment) attachment);
                } else if (attachment instanceof LargeAttachment) { // add folder
                    saveLargeFile(dir, (LargeAttachment) attachment, largeAttachmentTransport);
                }
            }
        } catch (Exception ex) {
            throw new RuntimeException("Error for " + message.getSMEVMetadata().getMessageIdentity().getMessageId(), ex);
        }
    }

    public MessageIdAttachmentDownloadTemplateImpl(File storeDir) {
        if (storeDir == null)
            throw new NullPointerException("storeDir");
        this.storeDir = storeDir;
    }
}
